@@ -106,7 +106,7 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
             // Fix is only for windows with opacity that aren't being computed at all
             if self.config.window.opacity < 1. || self.config.window.blur {
                 for (_id, route) in self.router.routes.iter_mut() {
-                    route.update_config(&self.config, &self.router.font_library);
+                    route.update_config(&self.config, &self.router.font_library, false);
 
                     route.request_redraw();
                 }
@@ -153,15 +153,18 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                 }
             }
             RioEventType::Rio(RioEvent::RenderRoute(route_id)) => {
-                if let Some(route) = self.router.routes.get_mut(&window_id) {
-                    if self.config.renderer.disable_unfocused_render
-                        && !route.window.is_focused
-                    {
-                        return;
-                    }
+                if self.config.renderer.strategy.is_event_based() {
+                    if let Some(route) = self.router.routes.get_mut(&window_id) {
+                        if self.config.renderer.disable_unfocused_render
+                            && !route.window.is_focused
+                        {
+                            return;
+                        }
 
-                    if route_id == route.window.screen.ctx().current_route() {
-                        let timer_id = TimerId::new(Topic::RenderRoute, window_id);
+                        // TODO: Optionally Allow check if is the current route
+                        // before schedule render. Using the following code below:
+                        // if route_id == route.window.screen.ctx().current_route() {
+                        let timer_id = TimerId::new(Topic::RenderRoute, route_id);
                         let event = EventPayload::new(
                             RioEventType::Rio(RioEvent::Render),
                             window_id,
@@ -179,10 +182,31 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                     }
                 }
             }
+            RioEventType::Rio(RioEvent::PrepareUpdateConfig) => {
+                let timer_id = TimerId::new(Topic::UpdateConfig, 0);
+                let event = EventPayload::new(
+                    RioEventType::Rio(RioEvent::UpdateConfig),
+                    window_id,
+                );
+
+                if !self.scheduler.scheduled(timer_id) {
+                    self.scheduler.schedule(
+                        event,
+                        Duration::from_millis(250),
+                        false,
+                        timer_id,
+                    );
+                }
+            }
             RioEventType::Rio(RioEvent::UpdateGraphicLibrary) => {
                 if let Some(route) = self.router.routes.get_mut(&window_id) {
-                    let mut terminal =
-                        route.window.screen.ctx().current().terminal.lock();
+                    let mut terminal = route
+                        .window
+                        .screen
+                        .context_manager
+                        .current_mut()
+                        .terminal
+                        .lock();
                     let graphics = terminal.graphics_take_queues();
                     drop(terminal);
                     if let Some(graphic_queues) = graphics {
@@ -234,7 +258,11 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                         }
                     }
 
-                    route.update_config(&self.config, &self.router.font_library);
+                    route.update_config(
+                        &self.config,
+                        &self.router.font_library,
+                        has_font_updates,
+                    );
                     route.window.configure_window(&self.config);
 
                     if let Some(error) = &config_error {
@@ -246,7 +274,7 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
             }
             RioEventType::Rio(RioEvent::Exit) => {
                 if let Some(route) = self.router.routes.get_mut(&window_id) {
-                    if self.config.confirm_before_quit {
+                    if cfg!(target_os = "macos") && self.config.confirm_before_quit {
                         route.confirm_quit();
                         route.request_redraw();
                     } else {
@@ -265,7 +293,7 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                         self.router.routes.remove(&window_id);
 
                         // Unschedule pending events.
-                        self.scheduler.unschedule_window(window_id);
+                        self.scheduler.unschedule_window(route_id);
 
                         if self.router.routes.is_empty() {
                             event_loop.exit();
@@ -289,21 +317,26 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                 }
             }
             RioEventType::Rio(RioEvent::PrepareRender(millis)) => {
-                let timer_id = TimerId::new(Topic::Render, window_id);
-                let event =
-                    EventPayload::new(RioEventType::Rio(RioEvent::Render), window_id);
-
-                if !self.scheduler.scheduled(timer_id) {
-                    self.scheduler.schedule(
-                        event,
-                        Duration::from_millis(millis),
-                        false,
-                        timer_id,
+                if let Some(route) = self.router.routes.get(&window_id) {
+                    let timer_id = TimerId::new(
+                        Topic::Render,
+                        route.window.screen.ctx().current_route(),
                     );
+                    let event =
+                        EventPayload::new(RioEventType::Rio(RioEvent::Render), window_id);
+
+                    if !self.scheduler.scheduled(timer_id) {
+                        self.scheduler.schedule(
+                            event,
+                            Duration::from_millis(millis),
+                            false,
+                            timer_id,
+                        );
+                    }
                 }
             }
             RioEventType::Rio(RioEvent::PrepareRenderOnRoute(millis, route_id)) => {
-                let timer_id = TimerId::new(Topic::RenderRoute, window_id);
+                let timer_id = TimerId::new(Topic::RenderRoute, route_id);
                 let event = EventPayload::new(
                     RioEventType::Rio(RioEvent::RenderRoute(route_id)),
                     window_id,
@@ -319,7 +352,7 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                 }
             }
             RioEventType::Rio(RioEvent::BlinkCursor(millis, route_id)) => {
-                let timer_id = TimerId::new(Topic::CursorBlinking, window_id);
+                let timer_id = TimerId::new(Topic::CursorBlinking, route_id);
                 let event = EventPayload::new(
                     RioEventType::Rio(RioEvent::CursorBlinkingChangeOnRoute(route_id)),
                     window_id,
@@ -352,8 +385,13 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
             }
             RioEventType::Rio(RioEvent::Scroll(scroll)) => {
                 if let Some(route) = self.router.routes.get_mut(&window_id) {
-                    let mut terminal =
-                        route.window.screen.ctx().current().terminal.lock();
+                    let mut terminal = route
+                        .window
+                        .screen
+                        .context_manager
+                        .current_mut()
+                        .terminal
+                        .lock();
                     terminal.scroll_display(scroll);
                     drop(terminal);
                 }
@@ -401,9 +439,10 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
             }
             RioEventType::Rio(RioEvent::TextAreaSizeRequest(format)) => {
                 if let Some(route) = self.router.routes.get_mut(&window_id) {
-                    let layout = route.window.screen.sugarloaf.layout();
+                    let dimension =
+                        route.window.screen.context_manager.current().dimension;
                     let text =
-                        format(crate::renderer::utils::terminal_dimensions(&layout));
+                        format(crate::renderer::utils::terminal_dimensions(&dimension));
                     route
                         .window
                         .screen
@@ -414,23 +453,36 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                 }
             }
             RioEventType::Rio(RioEvent::ColorRequest(index, format)) => {
-                // TODO: colors could be coming terminal as well
-                // if colors has been declaratively changed
-                // Rio doesn't cover this case yet.
-                //
-                // In the future should try first get
-                // from Crosswords then state colors
-                // screen.colors()[index] or screen.renderer.colors[index]
                 if let Some(route) = self.router.routes.get_mut(&window_id) {
-                    let color = route.window.screen.renderer.colors[index];
-                    let rgb = ColorRgb::from_color_arr(color);
+                    let terminal = route
+                        .window
+                        .screen
+                        .context_manager
+                        .current()
+                        .terminal
+                        .lock();
+                    let color: ColorRgb = match terminal.colors()[index] {
+                        Some(color) => ColorRgb::from_color_arr(color),
+                        // Ignore cursor color requests unless it was changed.
+                        None if index
+                            == crate::crosswords::NamedColor::Cursor as usize =>
+                        {
+                            return
+                        }
+                        None => ColorRgb::from_color_arr(
+                            route.window.screen.renderer.colors[index],
+                        ),
+                    };
+
+                    drop(terminal);
+
                     route
                         .window
                         .screen
                         .ctx_mut()
                         .current_mut()
                         .messenger
-                        .send_bytes(format(rgb).into_bytes());
+                        .send_bytes(format(color).into_bytes());
                 }
             }
             RioEventType::Rio(RioEvent::CreateWindow) => {
@@ -449,41 +501,34 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                     // it also reaches for the foreground process path if
                     // config.use_current_path is true
                     // For these case we need to make a workaround
-                    //
-                    // TODO: Reimplement this flow
-                    let mut should_revert_to_previous_config: Option<
-                        rio_backend::config::Config,
-                    > = None;
-                    if working_dir_overwrite.is_some() {
-                        let current_config = &self.config;
-                        should_revert_to_previous_config = Some(current_config.clone());
-
-                        let config = rio_backend::config::Config {
+                    let config = if working_dir_overwrite.is_some() {
+                        rio_backend::config::Config {
                             working_dir: working_dir_overwrite,
-                            ..current_config.clone()
-                        };
-                        self.config = config;
-                    }
+                            ..self.config.clone()
+                        }
+                    } else {
+                        self.config.clone()
+                    };
 
                     self.router.create_native_tab(
                         event_loop,
                         self.event_proxy.clone(),
-                        &self.config,
+                        &config,
                         Some(&route.window.winit_window.tabbing_identifier()),
                         None,
                     );
-
-                    if let Some(old_config) = should_revert_to_previous_config {
-                        self.config = old_config;
-                    }
                 }
             }
             RioEventType::Rio(RioEvent::CreateConfigEditor) => {
-                self.router.open_config_window(
-                    event_loop,
-                    self.event_proxy.clone(),
-                    &self.config,
-                );
+                if self.config.navigation.open_config_with_split {
+                    self.router.open_config_split(&self.config);
+                } else {
+                    self.router.open_config_window(
+                        event_loop,
+                        self.event_proxy.clone(),
+                        &self.config,
+                    );
+                }
             }
             #[cfg(target_os = "macos")]
             RioEventType::Rio(RioEvent::CloseWindow) => {
@@ -608,10 +653,18 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
 
         match event {
             WindowEvent::CloseRequested => {
-                self.router.routes.remove(&window_id);
+                // MacOS doesn't exit the loop
+                if cfg!(target_os = "macos") && self.config.confirm_before_quit {
+                    self.router.routes.remove(&window_id);
+                    return;
+                }
 
                 if self.config.confirm_before_quit {
+                    route.confirm_quit();
+                    route.request_redraw();
                     return;
+                } else {
+                    self.router.routes.remove(&window_id);
                 }
 
                 if self.router.routes.is_empty() {
@@ -628,7 +681,7 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                     }
 
                     route.window.winit_window.set_cursor(CursorIcon::Pointer);
-                    route.window.screen.context_manager.schedule_render(60);
+                    route.window.screen.context_manager.request_render();
                 }
             }
 
@@ -663,6 +716,9 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
 
                 match state {
                     ElementState::Pressed => {
+                        // In case need to switch grid current
+                        route.window.screen.select_current_based_on_mouse();
+
                         if route.window.screen.trigger_hyperlink() {
                             return;
                         }
@@ -800,7 +856,7 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                 let display_offset = route.window.screen.display_offset();
                 let old_point = route.window.screen.mouse_position(display_offset);
 
-                let layout = route.window.screen.sugarloaf.layout();
+                let layout = route.window.screen.sugarloaf.window_size();
 
                 let x = x.clamp(0.0, (layout.width as i32 - 1).into()) as usize;
                 let y = y.clamp(0.0, (layout.height as i32 - 1).into()) as usize;
@@ -824,7 +880,7 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
 
                 if route.window.screen.search_nearest_hyperlink_from_pos() {
                     route.window.winit_window.set_cursor(CursorIcon::Pointer);
-                    route.window.screen.context_manager.schedule_render(60);
+                    route.window.screen.context_manager.request_render();
                 } else {
                     let cursor_icon =
                         if !route.window.screen.modifiers.state().shift_key()
@@ -838,9 +894,20 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                     route.window.winit_window.set_cursor(cursor_icon);
 
                     // In case hyperlink range has cleaned trigger one more render
-                    if route.window.screen.renderer.has_hyperlink_range() {
-                        route.window.screen.renderer.set_hyperlink_range(None);
-                        route.window.screen.context_manager.schedule_render(60);
+                    if route
+                        .window
+                        .screen
+                        .context_manager
+                        .current()
+                        .has_hyperlink_range()
+                    {
+                        route
+                            .window
+                            .screen
+                            .context_manager
+                            .current_mut()
+                            .set_hyperlink_range(None);
+                        route.window.screen.context_manager.request_render();
                     }
                 }
 
@@ -852,7 +919,7 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                         || !route.window.screen.mouse_mode())
                 {
                     route.window.screen.update_selection(point, square_side);
-                    route.window.screen.context_manager.schedule_render(60);
+                    route.window.screen.context_manager.request_render();
                 } else if square_changed
                     && route.window.screen.has_mouse_motion_and_drag()
                 {
@@ -883,7 +950,7 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
 
                 match delta {
                     MouseScrollDelta::LineDelta(columns, lines) => {
-                        let layout = route.window.screen.sugarloaf.layout();
+                        let layout = route.window.screen.sugarloaf.rich_text_layout(&0);
                         let new_scroll_px_x = columns * layout.font_size;
                         let new_scroll_px_y = lines * layout.font_size;
                         route
@@ -925,13 +992,15 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                         && key_event.state == ElementState::Released
                     {
                         // Scheduler must be cleaned after leave the terminal route
-                        self.scheduler
-                            .unschedule(TimerId::new(Topic::Render, window_id));
+                        self.scheduler.unschedule(TimerId::new(
+                            Topic::Render,
+                            route.window.screen.ctx().current_route(),
+                        ));
                     }
                     return;
                 }
 
-                route.window.screen.renderer.last_typing = Some(Instant::now());
+                route.window.screen.context_manager.set_last_typing();
                 route.window.screen.process_key_event(&key_event);
 
                 if key_event.state == ElementState::Released
@@ -958,16 +1027,36 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                             Some(Preedit::new(text, cursor_offset.map(|offset| offset.0)))
                         };
 
-                        if route.window.screen.ime.preedit() != preedit.as_ref() {
-                            route.window.screen.ime.set_preedit(preedit);
+                        if route.window.screen.context_manager.current().ime.preedit()
+                            != preedit.as_ref()
+                        {
+                            route
+                                .window
+                                .screen
+                                .context_manager
+                                .current_mut()
+                                .ime
+                                .set_preedit(preedit);
                             route.request_redraw();
                         }
                     }
                     Ime::Enabled => {
-                        route.window.screen.ime.set_enabled(true);
+                        route
+                            .window
+                            .screen
+                            .context_manager
+                            .current_mut()
+                            .ime
+                            .set_enabled(true);
                     }
                     Ime::Disabled => {
-                        route.window.screen.ime.set_enabled(false);
+                        route
+                            .window
+                            .screen
+                            .context_manager
+                            .current_mut()
+                            .ime
+                            .set_enabled(false);
                     }
                 }
             }
@@ -980,12 +1069,12 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                     route.window.winit_window.set_cursor_visible(true);
                 }
 
-                // let has_regained_focus = !route.window.is_focused && focused;
-                // route.window.is_focused = focused;
+                let has_regained_focus = !route.window.is_focused && focused;
+                route.window.is_focused = focused;
 
-                // if has_regained_focus {
-                // route.request_redraw();
-                // }
+                if has_regained_focus {
+                    route.request_redraw();
+                }
 
                 route.window.screen.on_focus_change(focused);
             }
@@ -996,10 +1085,11 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
 
             WindowEvent::ThemeChanged(new_theme) => {
                 update_colors_based_on_theme(&mut self.config, Some(new_theme));
-                route
-                    .window
-                    .screen
-                    .update_config(&self.config, &self.router.font_library);
+                route.window.screen.update_config(
+                    &self.config,
+                    &self.router.font_library,
+                    false,
+                );
                 route.window.configure_window(&self.config);
             }
 
@@ -1055,6 +1145,13 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                 // println!("Time elapsed in render() is: {:?}", duration);
                 // }
 
+                if self.config.renderer.strategy.is_continuous()
+                    && !self.config.renderer.disable_unfocused_render
+                    && route.window.is_focused
+                {
+                    route.request_frame(&mut self.scheduler);
+                }
+
                 event_loop.set_control_flow(ControlFlow::Wait);
             }
             _ => {}
@@ -1073,11 +1170,15 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
     }
 
     fn open_config(&mut self, event_loop: &ActiveEventLoop) {
-        self.router.open_config_window(
-            event_loop,
-            self.event_proxy.clone(),
-            &self.config,
-        );
+        if self.config.navigation.open_config_with_split {
+            self.router.open_config_split(&self.config);
+        } else {
+            self.router.open_config_window(
+                event_loop,
+                self.event_proxy.clone(),
+                &self.config,
+            );
+        }
     }
 
     fn hook_event(&mut self, _event_loop: &ActiveEventLoop, hook: &Hook) {
@@ -1109,9 +1210,17 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                     route.window.screen.create_tab();
                 }
             }
-            Hook::CloseTab => {
-                if self.config.navigation.has_navigation_key_bindings() {
-                    route.window.screen.close_tab();
+            Hook::Close => {
+                route.window.screen.close_split_or_tab();
+            }
+            Hook::SplitDown => {
+                if self.config.navigation.use_split {
+                    route.window.screen.split_down();
+                }
+            }
+            Hook::SplitRight => {
+                if self.config.navigation.use_split {
+                    route.window.screen.split_right();
                 }
             }
         }
