@@ -11,7 +11,7 @@ use crate::crosswords::square::{Flags, Square};
 use crate::screen::hint::HintMatches;
 use navigation::ScreenNavigation;
 use rio_backend::config::colors::{
-    term::{List, TermColors},
+    term::{List, DIM_FACTOR},
     AnsiColor, ColorArray, Colors, NamedColor,
 };
 use rio_backend::config::Config;
@@ -28,10 +28,11 @@ use unicode_width::UnicodeWidthChar;
 
 pub struct Renderer {
     is_vi_mode_enabled: bool,
-    pub is_kitty_keyboard_enabled: bool,
+    draw_bold_text_with_light_colors: bool,
     pub named_colors: Colors,
     pub colors: List,
     pub navigation: ScreenNavigation,
+    unfocused_split_opacity: f32,
     pub config_has_blinking_enabled: bool,
     pub config_blinking_interval: u64,
     ignore_selection_fg_color: bool,
@@ -55,8 +56,7 @@ impl Renderer {
         config: &Config,
         font_context: &rio_backend::sugarloaf::font::FontLibrary,
     ) -> Renderer {
-        let term_colors = TermColors::default();
-        let colors = List::from(&term_colors);
+        let colors = List::from(&config.colors);
         let named_colors = config.colors;
 
         let mut dynamic_background =
@@ -80,10 +80,11 @@ impl Renderer {
         }
 
         Renderer {
+            unfocused_split_opacity: config.navigation.unfocused_split_opacity,
+            draw_bold_text_with_light_colors: config.draw_bold_text_with_light_colors,
             macos_use_unified_titlebar: config.window.macos_use_unified_titlebar,
             config_blinking_interval: config.cursor.blinking_interval.clamp(350, 1200),
             option_as_alt: config.option_as_alt.to_lowercase(),
-            is_kitty_keyboard_enabled: config.keyboard.use_kitty_keyboard_protocol,
             is_vi_mode_enabled: false,
             config_has_blinking_enabled: config.cursor.blinking,
             ignore_selection_fg_color: config.ignore_selection_fg_color,
@@ -107,11 +108,16 @@ impl Renderer {
     }
 
     #[inline]
-    fn create_style(&mut self, square: &Square) -> (FragmentStyle, char) {
+    fn create_style(
+        &mut self,
+        square: &Square,
+        renderable_content: &RenderableContent,
+    ) -> (FragmentStyle, char) {
         let flags = square.flags;
 
-        let mut foreground_color = self.compute_color(&square.fg, flags);
-        let mut background_color = self.compute_bg_color(square);
+        let mut foreground_color =
+            self.compute_color(&square.fg, flags, renderable_content);
+        let mut background_color = self.compute_bg_color(square, renderable_content);
 
         let content = if square.c == '\t' || flags.contains(Flags::HIDDEN) {
             ' '
@@ -144,7 +150,8 @@ impl Renderer {
             Some(background_color)
         };
 
-        let (decoration, decoration_color) = self.compute_decoration(square);
+        let (decoration, decoration_color) =
+            self.compute_decoration(square, renderable_content);
 
         (
             FragmentStyle {
@@ -163,6 +170,7 @@ impl Renderer {
     fn compute_decoration(
         &self,
         square: &Square,
+        renderable_content: &RenderableContent,
     ) -> (Option<FragmentStyleDecoration>, Option<[f32; 4]>) {
         let mut decoration = None;
         let mut decoration_color = None;
@@ -208,7 +216,8 @@ impl Renderer {
 
         if decoration.is_some() {
             if let Some(color) = square.underline_color() {
-                decoration_color = Some(self.compute_color(&color, square.flags));
+                decoration_color =
+                    Some(self.compute_color(&color, square.flags, renderable_content));
             }
         };
 
@@ -244,12 +253,13 @@ impl Renderer {
                 continue;
             }
 
-            let (mut style, square_content) =
-                if has_cursor && column == cursor.state.pos.col {
-                    self.create_cursor_style(square, cursor, is_active)
-                } else {
-                    self.create_style(square)
-                };
+            let (mut style, square_content) = if has_cursor
+                && column == cursor.state.pos.col
+            {
+                self.create_cursor_style(square, cursor, is_active, renderable_content)
+            } else {
+                self.create_style(square, renderable_content)
+            };
 
             if hyperlink_range.is_some()
                 && square.hyperlink().is_some()
@@ -270,19 +280,19 @@ impl Renderer {
                     .contains(Pos::new(line, Column(column)))
             {
                 style.color = if self.ignore_selection_fg_color {
-                    self.compute_color(&square.fg, square.flags)
+                    self.compute_color(&square.fg, square.flags, renderable_content)
                 } else {
                     self.named_colors.selection_foreground
                 };
                 style.background_color = Some(self.named_colors.selection_background);
             } else if search_hints.is_some()
-                && search_hints.as_mut().map_or(false, |search| {
-                    search.advance(Pos::new(line, Column(column)))
-                })
+                && search_hints
+                    .as_mut()
+                    .is_some_and(|search| search.advance(Pos::new(line, Column(column))))
             {
                 let is_focused = focused_match
                     .as_ref()
-                    .map_or(false, |fm| fm.contains(&Pos::new(line, Column(column))));
+                    .is_some_and(|fm| fm.contains(&Pos::new(line, Column(column))));
                 if is_focused {
                     style.color = self.named_colors.search_focused_match_foreground;
                     style.background_color =
@@ -294,6 +304,13 @@ impl Renderer {
                 }
             }
 
+            if !is_active {
+                style.color[3] = self.unfocused_split_opacity;
+                if let Some(mut background_color) = style.background_color {
+                    background_color[3] = self.unfocused_split_opacity;
+                }
+            }
+
             if square.flags.contains(Flags::GRAPHICS) {
                 // let graphics = square.graphics().map(|graphics| {
                 //     graphics
@@ -413,56 +430,37 @@ impl Renderer {
     }
 
     #[inline]
-    fn compute_color(&self, color: &AnsiColor, flags: Flags) -> ColorArray {
+    fn compute_color(
+        &self,
+        color: &AnsiColor,
+        flags: Flags,
+        renderable_content: &RenderableContent,
+    ) -> ColorArray {
         match color {
-            AnsiColor::Named(ansi_name) => match (ansi_name, flags) {
-                (NamedColor::Background, _) => self.named_colors.background.0,
-                (NamedColor::Cursor, _) => self.named_colors.cursor,
-
-                (NamedColor::Black, Flags::DIM) => self.named_colors.dim_black,
-                (NamedColor::Black, Flags::BOLD) => self.named_colors.light_black,
-                (NamedColor::Black, _) => self.named_colors.black,
-                (NamedColor::Blue, Flags::DIM) => self.named_colors.dim_blue,
-                (NamedColor::Blue, Flags::BOLD) => self.named_colors.light_blue,
-                (NamedColor::Blue, _) => self.named_colors.blue,
-                (NamedColor::Cyan, Flags::DIM) => self.named_colors.dim_cyan,
-                (NamedColor::Cyan, Flags::BOLD) => self.named_colors.light_cyan,
-                (NamedColor::Cyan, _) => self.named_colors.cyan,
-                (NamedColor::Foreground, _) => self.named_colors.foreground,
-                (NamedColor::Green, Flags::DIM) => self.named_colors.dim_green,
-                (NamedColor::Green, Flags::BOLD) => self.named_colors.light_green,
-                (NamedColor::Green, _) => self.named_colors.green,
-                (NamedColor::Magenta, Flags::DIM) => self.named_colors.dim_magenta,
-                (NamedColor::Magenta, Flags::BOLD) => self.named_colors.light_magenta,
-                (NamedColor::Magenta, _) => self.named_colors.magenta,
-                (NamedColor::Red, Flags::DIM) => self.named_colors.dim_red,
-                (NamedColor::Red, Flags::BOLD) => self.named_colors.light_red,
-                (NamedColor::Red, _) => self.named_colors.red,
-                (NamedColor::White, Flags::DIM) => self.named_colors.dim_white,
-                (NamedColor::White, Flags::BOLD) => self.named_colors.light_white,
-                (NamedColor::White, _) => self.named_colors.white,
-                (NamedColor::Yellow, Flags::DIM) => self.named_colors.dim_yellow,
-                (NamedColor::Yellow, Flags::BOLD) => self.named_colors.light_yellow,
-                (NamedColor::Yellow, _) => self.named_colors.yellow,
-                (NamedColor::LightBlack, _) => self.named_colors.light_black,
-                (NamedColor::LightBlue, _) => self.named_colors.light_blue,
-                (NamedColor::LightCyan, _) => self.named_colors.light_cyan,
-                (NamedColor::LightForeground, _) => self.named_colors.light_foreground,
-                (NamedColor::LightGreen, _) => self.named_colors.light_green,
-                (NamedColor::LightMagenta, _) => self.named_colors.light_magenta,
-                (NamedColor::LightRed, _) => self.named_colors.light_red,
-                (NamedColor::LightWhite, _) => self.named_colors.light_white,
-                (NamedColor::LightYellow, _) => self.named_colors.light_yellow,
-                (NamedColor::DimBlack, _) => self.named_colors.dim_black,
-                (NamedColor::DimBlue, _) => self.named_colors.dim_blue,
-                (NamedColor::DimCyan, _) => self.named_colors.dim_cyan,
-                (NamedColor::DimForeground, _) => self.named_colors.dim_foreground,
-                (NamedColor::DimGreen, _) => self.named_colors.dim_green,
-                (NamedColor::DimMagenta, _) => self.named_colors.dim_magenta,
-                (NamedColor::DimRed, _) => self.named_colors.dim_red,
-                (NamedColor::DimWhite, _) => self.named_colors.dim_white,
-                (NamedColor::DimYellow, _) => self.named_colors.dim_yellow,
-            },
+            AnsiColor::Named(ansi) => {
+                match (
+                    self.draw_bold_text_with_light_colors,
+                    flags & Flags::DIM_BOLD,
+                ) {
+                    // If no bright foreground is set, treat it like the BOLD flag doesn't exist.
+                    (_, Flags::DIM_BOLD)
+                        if ansi == &NamedColor::Foreground
+                            && self.named_colors.light_foreground.is_none() =>
+                    {
+                        self.color(NamedColor::DimForeground as usize, renderable_content)
+                    }
+                    // Draw bold text in bright colors *and* contains bold flag.
+                    (true, Flags::BOLD) => {
+                        self.color(ansi.to_light() as usize, renderable_content)
+                    }
+                    // Cell is marked as dim and not bold.
+                    (_, Flags::DIM) | (false, Flags::DIM_BOLD) => {
+                        self.color(ansi.to_dim() as usize, renderable_content)
+                    }
+                    // None of the above, keep original color..
+                    _ => self.color(*ansi as usize, renderable_content),
+                }
+            }
             AnsiColor::Spec(rgb) => {
                 if !flags.contains(Flags::DIM) {
                     rgb.to_arr()
@@ -479,64 +477,39 @@ impl Renderer {
                     _ => *index as usize,
                 };
 
-                self.colors[index]
+                self.color(index, renderable_content)
             }
         }
     }
 
     #[inline]
-    fn compute_bg_color(&self, square: &Square) -> ColorArray {
+    fn compute_bg_color(
+        &self,
+        square: &Square,
+        renderable_content: &RenderableContent,
+    ) -> ColorArray {
         match square.bg {
-            AnsiColor::Named(ansi_name) => match (ansi_name, square.flags) {
-                (NamedColor::Background, _) => self.named_colors.background.0,
-                (NamedColor::Cursor, _) => self.named_colors.cursor,
-
-                (NamedColor::Black, Flags::DIM) => self.named_colors.dim_black,
-                (NamedColor::Black, Flags::BOLD) => self.named_colors.light_black,
-                (NamedColor::Black, _) => self.named_colors.black,
-                (NamedColor::Blue, Flags::DIM) => self.named_colors.dim_blue,
-                (NamedColor::Blue, Flags::BOLD) => self.named_colors.light_blue,
-                (NamedColor::Blue, _) => self.named_colors.blue,
-                (NamedColor::Cyan, Flags::DIM) => self.named_colors.dim_cyan,
-                (NamedColor::Cyan, Flags::BOLD) => self.named_colors.light_cyan,
-                (NamedColor::Cyan, _) => self.named_colors.cyan,
-                (NamedColor::Foreground, _) => self.named_colors.foreground,
-                (NamedColor::Green, Flags::DIM) => self.named_colors.dim_green,
-                (NamedColor::Green, Flags::BOLD) => self.named_colors.light_green,
-                (NamedColor::Green, _) => self.named_colors.green,
-                (NamedColor::Magenta, Flags::DIM) => self.named_colors.dim_magenta,
-                (NamedColor::Magenta, Flags::BOLD) => self.named_colors.light_magenta,
-                (NamedColor::Magenta, _) => self.named_colors.magenta,
-                (NamedColor::Red, Flags::DIM) => self.named_colors.dim_red,
-                (NamedColor::Red, Flags::BOLD) => self.named_colors.light_red,
-                (NamedColor::Red, _) => self.named_colors.red,
-                (NamedColor::White, Flags::DIM) => self.named_colors.dim_white,
-                (NamedColor::White, Flags::BOLD) => self.named_colors.light_white,
-                (NamedColor::White, _) => self.named_colors.white,
-                (NamedColor::Yellow, Flags::DIM) => self.named_colors.dim_yellow,
-                (NamedColor::Yellow, Flags::BOLD) => self.named_colors.light_yellow,
-                (NamedColor::Yellow, _) => self.named_colors.yellow,
-                (NamedColor::LightBlack, _) => self.named_colors.light_black,
-                (NamedColor::LightBlue, _) => self.named_colors.light_blue,
-                (NamedColor::LightCyan, _) => self.named_colors.light_cyan,
-                (NamedColor::LightForeground, _) => self.named_colors.light_foreground,
-                (NamedColor::LightGreen, _) => self.named_colors.light_green,
-                (NamedColor::LightMagenta, _) => self.named_colors.light_magenta,
-                (NamedColor::LightRed, _) => self.named_colors.light_red,
-                (NamedColor::LightWhite, _) => self.named_colors.light_white,
-                (NamedColor::LightYellow, _) => self.named_colors.light_yellow,
-                (NamedColor::DimBlack, _) => self.named_colors.dim_black,
-                (NamedColor::DimBlue, _) => self.named_colors.dim_blue,
-                (NamedColor::DimCyan, _) => self.named_colors.dim_cyan,
-                (NamedColor::DimForeground, _) => self.named_colors.dim_foreground,
-                (NamedColor::DimGreen, _) => self.named_colors.dim_green,
-                (NamedColor::DimMagenta, _) => self.named_colors.dim_magenta,
-                (NamedColor::DimRed, _) => self.named_colors.dim_red,
-                (NamedColor::DimWhite, _) => self.named_colors.dim_white,
-                (NamedColor::DimYellow, _) => self.named_colors.dim_yellow,
+            AnsiColor::Named(ansi) => self.color(ansi as usize, renderable_content),
+            AnsiColor::Spec(rgb) => match square.flags & Flags::DIM {
+                Flags::DIM => (&(rgb * DIM_FACTOR)).into(),
+                _ => (&rgb).into(),
             },
-            AnsiColor::Spec(rgb) => rgb.to_arr(),
-            AnsiColor::Indexed(idx) => self.colors[idx as usize],
+            AnsiColor::Indexed(idx) => {
+                let idx = match (
+                    self.draw_bold_text_with_light_colors,
+                    square.flags & Flags::DIM_BOLD,
+                    idx,
+                ) {
+                    (true, Flags::BOLD, 0..=7) => idx as usize + 8,
+                    (false, Flags::DIM, 8..=15) => idx as usize - 8,
+                    (false, Flags::DIM, 0..=7) => {
+                        NamedColor::DimBlack as usize + idx as usize
+                    }
+                    _ => idx as usize,
+                };
+
+                self.color(idx, renderable_content)
+            }
         }
     }
 
@@ -546,6 +519,7 @@ impl Renderer {
         square: &Square,
         cursor: &Cursor,
         is_active: bool,
+        renderable_content: &RenderableContent,
     ) -> (FragmentStyle, char) {
         let font_attrs = match (
             square.flags.contains(Flags::ITALIC),
@@ -558,8 +532,8 @@ impl Renderer {
             _ => (Stretch::NORMAL, Weight::NORMAL, Style::Normal),
         };
 
-        let mut color = self.compute_color(&square.fg, square.flags);
-        let mut background_color = self.compute_bg_color(square);
+        let mut color = self.compute_color(&square.fg, square.flags, renderable_content);
+        let mut background_color = self.compute_bg_color(square, renderable_content);
         // If IME is enabled we get the current content to cursor
         let content = if cursor.is_ime_enabled {
             cursor.content
@@ -606,12 +580,14 @@ impl Renderer {
         };
 
         let cursor_color = if !self.is_vi_mode_enabled {
-            self.named_colors.cursor
+            renderable_content.colors[NamedColor::Cursor]
+                .unwrap_or(self.named_colors.cursor)
         } else {
             self.named_colors.vi_cursor
         };
 
-        let (decoration, decoration_color) = self.compute_decoration(square);
+        let (decoration, decoration_color) =
+            self.compute_decoration(square, renderable_content);
         style.decoration = decoration;
         style.decoration_color = decoration_color;
 
@@ -648,6 +624,16 @@ impl Renderer {
         self.is_vi_mode_enabled = is_vi_mode_enabled;
     }
 
+    // Get the RGB value for a color index.
+    #[inline]
+    pub fn color(
+        &self,
+        color: usize,
+        renderable_content: &RenderableContent,
+    ) -> ColorArray {
+        renderable_content.colors[color].unwrap_or(self.colors[color])
+    }
+
     #[inline]
     pub fn prepare_term(
         &mut self,
